/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2023 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
const express = require('express');
const moment = require('moment');

const systemPlugin = require('./utils/plugins/systemPlugin.js');
const utils = require('./utils/utils');
const permissions = require("./utils/permissions/permissions.js");
const EmbedableDashboard = require('../ODM/models.js').embedableDashboard;
const QueryStage = require('../../analytics/engine/stages/QueryStage');
const Executor = require('../../analytics/engine/Executor');
const fetch = require('node-fetch');

module.exports = class SenseDashboards extends systemPlugin {
    constructor (odm, settings) {
        super(odm, settings);

        this.on('adminTools', (event) => ({
            text: 'Sen$e Dashboards',
            href: this.getLink('/sense-dashboards/'),
            subtext: 'Sense analytics derived from xAPI statements in the LRS.',
        }));

        const router = express.Router();
        router.use(utils.ensureLoggedIn);
        router.use(permissions.permsMiddleware('lrs.analytics.view', null));

        const padTo2Digits = (num) => num.toString().padStart(2, '0');
        const padTo4Digits = (num) => num.toString().padStart(4, '0');
        
        const getDateParams = async (lrsUuid) => {
            const dateQuery = {
                filter: { },
                process: [
                    {
                        "$group": {
                            "_id": { "$year": "$statement.timestamp"},
                            "stmts": { "$sum": 1 }
                        }
                    }
                ]
            };

            const queryStage = new QueryStage({
                filter: dateQuery.filter,
                process: dateQuery.process,
                lrs: { uuid: lrsUuid },
            });
            const dateResults = await Executor.runStage(queryStage, null, { uuid: lrsUuid });
            // list of years
            let years = dateResults
                .map(r => [padTo4Digits(r._id - 1), padTo4Digits(r._id), padTo4Digits(r._id + 1)])
                .flat();
            years = [...new Set(years)];
            years.sort().reverse();
            const currYear = (new Date()).getFullYear();
            if (parseInt(years[0]) > currYear) years.shift();


            return {
                months: moment.months().map((month, i) => ({ value: padTo2Digits(i + 1), display: month })),
                years: years,
            }
        }

        router.get('/uptime', async (req, res) => {
            let uptime;

            try {
                const to = new Date(req.query.to).getTime();
                const from = new Date(req.query.from).getTime();
                console.log('url', `${settings.uptimeApiUrl}v2/getMonitors?api_key=${settings.uptimeApiKey} ${to} ${from}`)

                const response = await fetch(`${settings.uptimeApiUrl}v2/getMonitors?api_key=${settings.uptimeApiKey}&monitors=${settings.uptimeMonitor}&custom_uptime_ranges=${Math.floor(from / 1000)}_${Math.floor(to / 1000)}`, {
                    method: 'POST',
                    headers: {
                        Accept: 'application/json',
                        'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({
                        monitors: settings.uptimeMonitor,
                        custom_uptime_ranges: `${Math.floor(from / 1000)}_${Math.floor(to / 1000)}`,
                    }),
                });

                const data = await response.json();
                console.log('data', JSON.stringify(data, null, 2))

                if (data.stat !== 'ok') throw new Error(data.error.message);
                console.log('monitor', data.monitors.find(m => m.id === parseInt(settings.uptimeMonitor)))
                uptime = data.monitors.find(m => m.id === parseInt(settings.uptimeMonitor)).custom_uptime_ranges;
            } catch (err) {
                console.log('Error ', err);
                return res.status(400).send({
                    uptime: 'There was an error in connecting to the remote uptime monitor.',
                });
            }

            res.send({
                uptime: parseFloat(uptime).toFixed(2),
                // uptime: 82
            });
        })

        router.get('/sense-dashboards', async (req, res) => {
            // Get the usage dashboard available months and years.
            //  Get the LRS for the usage dashboard
            const kpiDashboard = await EmbedableDashboard.findOne({ uuid: settings.kpiDashboard });
            const kpiLRS = kpiDashboard.lrs;

            // Get available dates
            const dates = await getDateParams(kpiLRS);

            req.session.allowedDashboards = [
                settings.kpiDashboard,
                settings.usageDashboard,
                settings.executiveDashboard
            ];

            return res.render('templates/plugins/sense/index', {
                kpiDashboard: settings.kpiDashboard,
                usageDashboard: settings.usageDashboard,
                executiveDashboard: settings.executiveDashboard,
                ...dates,
                user: req.user
            });
        });

        this.setRouter('system', router);
    }

    static get display() {
        return {
            title: 'Sen$e Dashboards',
            description: 'A plugin for viewing Sen$e Dashboards.',
        };
    }

    // Install will be run when the plugin is added into an LRS
    async install() {
        logger.debug(`installed, path at ${this.getLink('/sense-dashboards')}`);
    }
    // Uninstall will be run when the plugin is removed from the LRS
    async uninstall() {
        logger.debug('uninstall me ' + this.uuid);
        // We need to make sure to call the superclass uninstall, or we'll leave behind queued events!
        super.uninstall();
    }

    // Additional metadata for display
    static get metadata() {
        return {
            author: 'Veracity Technology Consultants',
            version: '1.0.0',
            moreInfo: 'https://www.veracity.it',
        };
    }

    static get settingsForm() {
        return [
            {
                id: 'usageDashboard',
                label: 'Usage Dasboard UUID',
                type: { isText: true },
                helptext: 'The UUID of the exported Usage Dashboard link.'
            },
            {
                id: 'kpiDashboard',
                label: 'KPI Dasboard UUID',
                type: { isText: true },
                helptext: 'The UUID of the exported KPI Dashboard link.'
            },
            {
                id: 'executiveDashboard',
                label: 'Executive Dasboard UUID',
                type: { isText: true },
                helptext: 'The UUID of the exported Executive Dashboard link.'
            },
            {
                id: 'uptimeApiUrl',
                label: 'Uptime Monitor API URL',
                type: { isText: true },
                helptext: 'The root URL of the uptime monitor api.'
            },
            {
                id: 'uptimeApiKey',
                label: 'Uptime Monitor API Key',
                type: { isText: true, type: 'password' },
                helptext: 'The API Key for the uptime monitor api.'
            },
            {
                id: 'uptimeMonitor',
                label: 'Uptime Monitor Code',
                type: { isText: true },
                helptext: 'The the code for the Sen$e application uptime monitor.'
            }
        ];
    }
};



module.exports.pluginName = 'Sen$e Dashboards';
